#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
"""
Tests for the functionality of the Notification REST API.
The logic can be found in `notification/views/notify.py`.
"""


# pylint: disable=C0415, E0401, R0201

# TODO: re-enable after tests are fixed
# pylint: disable=W0511

# TODO: re-enable after pydoc has been added
# pylint: disable=C0116

import http
from unittest.mock import patch

import pytest
from pyramid.testing import DummyRequest


@pytest.mark.usefixtures("dummy_send", "mock_notification_service")
class TestNotifyViews:
    """Unit tests for notification endpoints"""

    def test_view_notifications(self, dummy_send: DummyRequest):
        from notification.views.notify import NotificationListRestService

        response = NotificationListRestService(dummy_send).list_notification_templates()
        assert len(response) == 1
        # TODO: create a mock list of notification templates, and make sure that they're all found

    def test_send_notification(self, dummy_send: DummyRequest):
        from notification.views.notify import NotificationRestService

        dummy_send.json_body = {
            "destination_email": "chausman@nrao.edu",
            "subject": "first thing",
            "message": "second thing",
        }

        dummy_send.matchdict["template_name"] = "email"
        with patch(
            "workspaces.notification.services.notification_service.NotificationService.send_email"
        ) as send_email:
            send_email.return_value = {}
            response = NotificationRestService(dummy_send).send_notification()
        assert response["errors"] == {}
        assert response["message"] == "Email sent"

    def test_send_notification_with_list(self, dummy_send: DummyRequest):
        from notification.views.notify import NotificationRestService

        # TODO: save original dummy_send values, and after assertions restore them in a `finally()`

        dummy_send.json_body = {
            "destination_email": ["chausman@nrao.edu", "dreamerofthestars@gmail.com"],
            "subject": "first thing",
            "message": "second thing",
        }

        dummy_send.matchdict["template_name"] = "email"
        with patch(
            "workspaces.notification.services.notification_service.NotificationService.send_email"
        ) as send_email:
            send_email.return_value = {}
            response = NotificationRestService(dummy_send).send_notification()
        assert response["errors"] == {}
        assert response["message"] == "Email sent"

    @pytest.mark.skip("breaks update-template test")
    def test_create_notification(self, dummy_send: DummyRequest):
        # FIXME
        from notification.views.notify import NotificationListRestService

        # TODO: save original dummy_send.matchdict first, then restore it in a `finally:`
        dummy_send.json_body = {
            "name": "new_template",
            "description": "a test template",
            "content": "Subject: {{foo}}\n\n{{bar}}",
        }

        response = NotificationListRestService(dummy_send).create_notification_template()
        assert response.status_code == http.HTTPStatus.CREATED
        assert response.json_body == dummy_send.json_body

    def test_delete_notification(self, dummy_send: DummyRequest):
        from notification.views.notify import NotificationRestService

        dummy_send.matchdict["template_name"] = "new_template"

        response = NotificationRestService(dummy_send).delete_notification_template()
        assert response.status_code == http.HTTPStatus.OK
        # TODO: was template -actually- deleted?

    def test_update_notification_template(self, dummy_send: DummyRequest):
        from notification.views.notify import NotificationRestService

        dummy_send.json_body = {
            "template_name": "a_template",
            "template_text": '{"message": "this is the message", "routing_key": "ingestion-complete.#", '
            '"project_code": '
            '"13B-014"}',
        }

        response = NotificationRestService(dummy_send).update_notification_template()
        assert response.status_code == http.HTTPStatus.OK
        # TODO: did template text -really- get updated?
